/*  OpenTally: Open-source election vote counting
 *  Copyright © 2021 Lee Yingtong Li (RunasSudo)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#![allow(missing_docs)]

use pest::iterators::Pair;

#[derive(Parser)]
#[grammar = "parser/blt.pest"]
pub struct BLTParser;

/// Convert the specified [Pair<Rule>] into a &str
pub fn unwrap_string(string: Pair<Rule>) -> &str {
	match string.as_rule() {
		Rule::string => {
			let string = string.into_inner().next().unwrap();
			match string.as_rule() {
				Rule::raw_string => {
					return string.as_str();
				}
				Rule::quoted_string => {
					let string = string.as_str();
					return &string[1..string.len()-1];
				}
				_ => unreachable!()
			}
		}
		_ => panic!("Non-string passed to unwrap_string")
	}
}
