/*  OpenTally: Open-source election vote counting
 *  Copyright © 2021 Lee Yingtong Li (RunasSudo)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

mod utils;

use opentally::numbers::Rational;
use opentally::stv;

#[test]
fn ers97_rational() {
	let stv_opts = stv::STVOptions {
		round_tvs: Some(2),
		round_weights: Some(2),
		round_votes: Some(2),
		round_quota: Some(2),
		sum_surplus_transfers: stv::SumSurplusTransfersMode::SingleStep,
		meek_surplus_tolerance: String::new(),
		normalise_ballots: false,
		quota: stv::QuotaType::DroopExact,
		quota_criterion: stv::QuotaCriterion::GreaterOrEqual,
		quota_mode: stv::QuotaMode::ERS97,
		ties: vec![],
		surplus: stv::SurplusMethod::EG,
		surplus_order: stv::SurplusOrder::BySize,
		transferable_only: true,
		exclusion: stv::ExclusionMethod::ByValue,
		meek_nz_exclusion: false,
		early_bulk_elect: false,
		bulk_exclude: true,
		defer_surpluses: true,
		meek_immediate_elect: false,
		constraints_path: None,
		constraint_mode: stv::ConstraintMode::GuardDoom,
		pp_decimals: 2,
	};
	utils::read_validate_election::<Rational>("tests/data/ers97.csv", "tests/data/ers97.blt", stv_opts, None, &["nt", "vre"]);
}
